"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@oclif/core");
const yaml = require("yaml");
const fs = require("node:fs");
const fs_utils_1 = require("../lib/fs/fs-utils");
const profile_client_1 = require("../lib/spec/aml/profile-client");
const linter_1 = require("../lib/commands/linter");
class LintCore {
    constructor(fs, logger) {
        this.fs = fs;
        this.logger = logger;
    }
    async run(flags) {
        var _a;
        const location = (_a = flags.location) !== null && _a !== void 0 ? _a : '.';
        const profiles = fs_utils_1.default.listRuleProfiles(this.fs, location);
        const results = await Promise.all(profiles.map(profile => profile_client_1.default.parseRuleset(profile, this.fs)));
        let failure = false;
        const linter = new linter_1.default();
        for (const result of results) {
            const profile = yaml.parse(fs.readFileSync(result.source).toString());
            const termErrors = linter.lintProfileTerms(profile);
            const definitionResult = linter.checkRuleDefinitions(profile);
            if (result.conforms()) {
                if (definitionResult.success) {
                    this.logger.log(`\u001B[92m\u2713 profile ${result.location()} is conformant\u001B[0m`);
                }
                else {
                    this.logger.log(`\u001B[91m\u2717 profile ${result.location()} found a problem with the validation: ${definitionResult.message}\u001B[0m`);
                    failure = true;
                }
            }
            else {
                this.logger.log(`\u001B[91m\u2717 profile ${result.location()} is non conformant\u001B[0m`);
                this.logger.log(result.reportText());
                failure = true;
            }
            for (const term of Object.keys(termErrors)) {
                const error = termErrors[term];
                if (error.isRegex) {
                    this.logger.log(`  \u001B[33m! ${error.source} should be defined without initial and final '/'\u001B[0m`);
                }
                else {
                    const message = linter.termErrorMessage(error);
                    this.logger.log(`  \u001B[33m! ${message}\u001B[0m`);
                }
            }
        }
        if (failure) {
            this.logger.error('Found non-conformant profiles');
        }
    }
}
exports.default = LintCore;
LintCore.description = 'Validates the rules syntax without testing any examples';
LintCore.examples = [
    '<%= config.bin %> <%= command.id %>',
    '<%= config.bin %> <%= command.id %> -l /development/my-ruleset-project',
];
LintCore.flags = {
    location: core_1.Flags.string({ char: 'l', description: 'path where the project will be created' }),
};
LintCore.args = [];
//# sourceMappingURL=lint.js.map