"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const default_fs_1 = require("../fs/default-fs");
const yaml = require("yaml");
const config_loader_1 = require("../config-loader");
const fs_utils_1 = require("../fs/fs-utils");
const profile_client_1 = require("../spec/aml/profile-client");
class Build {
    constructor(location, logger, fs) {
        this.location = location;
        this.logger = logger;
        this.fs = fs ? fs : new default_fs_1.default();
    }
    async build() {
        const config = config_loader_1.default.fromFile(this.fs, this.location);
        const documentation = fs_utils_1.default.loadDocFile(this.fs, this.location);
        const ruleset = this.consolidateRuleset(config, documentation);
        const text = this.rulesetToString(ruleset);
        const profileFile = fs_utils_1.default.writeConsolidatedRuleset(this.fs, text, this.location);
        this.logger.log('* Generated consolidated ruleset at ' + profileFile);
        await this.lintProfile(profileFile);
    }
    async lintProfile(profileFile) {
        const lintResult = await profile_client_1.default.parseRuleset(profileFile, this.fs);
        if (lintResult.conforms()) {
            this.logger.log('* Consolidated ruleset linted successfully');
        }
        else {
            this.logger.log(lintResult.reportText());
            this.logger.error('* Error linting generated ruleset');
        }
    }
    rulesetToString(ruleset) {
        return '#%Validation Profile 1.0\n' + yaml.stringify(ruleset);
    }
    consolidateRuleset(config, documentation) {
        const rulesetName = config.name;
        const { acc, validations, tags, prefixes } = this.accumulatedRules();
        return this.buildRuleset(rulesetName, documentation, acc, validations, tags, config, prefixes);
    }
    buildRuleset(rulesetName, documentation, acc, validations, tags, config, prefixes) {
        const ruleset = {
            profile: rulesetName,
        };
        if (documentation !== undefined && documentation.length > 0)
            // eslint-disable-next-line @typescript-eslint/ban-ts-comment
            // @ts-ignore
            ruleset.description = documentation;
        if (acc.violation.length > 0) {
            // eslint-disable-next-line @typescript-eslint/ban-ts-comment
            // @ts-ignore
            ruleset.violation = acc.violation;
        }
        if (acc.warning.length > 0) {
            // eslint-disable-next-line @typescript-eslint/ban-ts-comment
            // @ts-ignore
            ruleset.warning = acc.warning;
        }
        if (acc.info.length > 0) {
            // eslint-disable-next-line @typescript-eslint/ban-ts-comment
            // @ts-ignore
            ruleset.info = acc.info;
        }
        tags.push(...this.getRulesetTagsFromConfig(config));
        if (tags.length > 0) {
            // eslint-disable-next-line @typescript-eslint/ban-ts-comment
            // @ts-ignore
            ruleset.tags = tags;
        }
        if (Object.keys(prefixes).length > 0) {
            // eslint-disable-next-line @typescript-eslint/ban-ts-comment
            // @ts-ignore
            ruleset.prefixes = prefixes;
        }
        // eslint-disable-next-line @typescript-eslint/ban-ts-comment
        // @ts-ignore
        ruleset.validations = validations;
        return ruleset;
    }
    getRulesetTagsFromConfig(config) {
        if (config.rulesetConfig !== undefined && config.rulesetConfig.tags.length > 0) {
            // eslint-disable-next-line @typescript-eslint/ban-ts-comment
            // @ts-ignore
            return config.rulesetConfig.tags;
        }
        return [];
    }
    addObjects(objA, objB) {
        return { ...objA, ...objB };
    }
    accumulatedRules() {
        const validations = {};
        const acc = {
            violation: new Array(),
            warning: new Array(),
            info: new Array(),
        };
        const tags = new Array();
        let accPrefixes = {};
        for (const ruleDir of fs_utils_1.default.listRuleDirs(this.fs, this.logger, this.location)) {
            const name = ruleDir.ruleName;
            const severity = ruleDir.severityLevel;
            const body = ruleDir.ruleBody;
            // eslint-disable-next-line @typescript-eslint/ban-ts-comment
            // @ts-ignore
            acc[severity].push(name);
            if (ruleDir.tags) {
                tags.push(...ruleDir.tags);
            }
            if (ruleDir.prefixes) {
                accPrefixes = this.addObjects(ruleDir.prefixes, accPrefixes);
            }
            validations[name] = body;
        }
        return {
            acc: acc,
            validations: validations,
            tags: tags,
            prefixes: accPrefixes,
        };
    }
}
exports.default = Build;
//# sourceMappingURL=build.js.map