"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const amf_model_1 = require("../model/amf-model");
class Linter {
    termErrorMessage(error) {
        const where = error.isClass ? 'class term' : 'path property';
        return error.score > 0.7 ? `unknown ${where} '${error.source}' did you mean '${error.term}'` : `unknown ${where} '${error.source}'`;
    }
    lintProfileTerms(profile) {
        const amfModel = new amf_model_1.AmfModel(profile.prefixes || {});
        return this.lintTraverse(profile, amfModel);
    }
    checkRuleDefinitions(profileNode) {
        const validations = profileNode.validations;
        if (!validations) {
            return {
                success: false,
                message: 'Profile has no defined validations',
            };
        }
        if (Object.keys(validations).length > 1) {
            return {
                success: false,
                message: 'More than one rule validation defined in the profile, you should define only 1 validation per test ruleset',
            };
        }
        const validation = Object.keys(validations)[0];
        const acc = [];
        if (profileNode.violation) {
            for (const v of profileNode.violation) {
                acc.push(v);
            }
        }
        if (profileNode.info) {
            for (const v of profileNode.info) {
                acc.push(v);
            }
        }
        if (profileNode.warning) {
            for (const v of profileNode.warning) {
                acc.push(v);
            }
        }
        if (acc.length === 0) {
            return {
                success: false,
                message: 'No validation assigned to any severity level, you should assign the only validation defined in the profile to some severity level in a test ruleset',
            };
        }
        if (acc.length > 1) {
            return {
                success: false,
                message: 'More than one violation assigned to a severity level, you should only define one validation and assigned to a severity level in a test ruleset',
            };
        }
        if (acc[0] !== validation) {
            return {
                success: false,
                message: `The defined validation '${validation}' does not match the validation assigned to a severity level with name '${acc[0]}'`,
            };
        }
        return {
            success: true,
            message: '',
        };
    }
    lintTraverse(profileNode, amfModel, acc = {}) {
        if (typeof profileNode === 'object') {
            for (const p of Object.keys(profileNode)) {
                switch (p) {
                    case 'pattern': {
                        const regex = profileNode[p];
                        if (regex.startsWith('/') && regex.endsWith('/')) {
                            acc[regex] = {
                                source: regex,
                                term: regex,
                                score: 1,
                                match: false,
                                isClass: false,
                                isProperty: false,
                                isRegex: true,
                            };
                        }
                        break;
                    }
                    case 'targetClass': {
                        const toCheck = profileNode[p];
                        this.lintClassTerm(toCheck, amfModel, acc);
                        break;
                    }
                    case 'propertyConstraints': {
                        const propertyConstraintNode = profileNode[p];
                        for (const prop of Object.keys(propertyConstraintNode)) {
                            this.lintPropertyPathTerm(prop, amfModel, acc);
                            this.lintTraverse(propertyConstraintNode[prop], amfModel, acc);
                        }
                        break;
                    }
                    default:
                        this.lintTraverse(profileNode[p], amfModel, acc);
                }
            }
        }
        return acc;
    }
    lintClassTerm(toCheck, amfModel, acc) {
        const match = amfModel.findClassMatch(toCheck);
        if (!match.match) {
            acc[match.source] = match;
        }
        return acc;
    }
    lintPropertyPathTerm(toCheck, amfModel, acc) {
        if (toCheck !== '@type') {
            const matches = amfModel.findPropertyMatch(toCheck);
            for (const match of matches) {
                if (!match.match) {
                    acc[match.source] = match;
                }
            }
        }
        return acc;
    }
}
exports.default = Linter;
//# sourceMappingURL=linter.js.map