"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const validate_1 = require("../validate");
const validator_report_1 = require("../validator-report");
const rule_tester_result_1 = require("./rule-tester-result");
const promises_utils_1 = require("../utils/promises-utils");
const fs_utils_1 = require("../fs/fs-utils");
const yaml = require("yaml");
const fs = require("node:fs");
const spec_client_provider_1 = require("../spec/spec-client-provider");
const profile_client_1 = require("../spec/aml/profile-client");
const linter_1 = require("./linter");
class RulesetTester {
    // eslint-disable-next-line max-params
    constructor(ruleDir, debug, lexicalInfo, filter, logger, skipValidation, fs) {
        this.ruleDir = ruleDir;
        this.debug = debug;
        this.filter = filter;
        this.logger = logger;
        this.skipValidation = skipValidation;
        this.fs = fs;
        this.lexicalInfo = lexicalInfo;
    }
    shouldValidate(name) {
        const re = new RegExp(this.filter);
        return re.test(name);
    }
    getExamples() {
        const allExamples = [...this.ruleDir.positive, ...this.ruleDir.negative];
        return allExamples.filter(f => {
            return this.shouldValidate(f);
        });
    }
    async parseInput(filePath) {
        const spectype = fs_utils_1.default.findSpecType(this.fs, filePath);
        if (spectype === spec_client_provider_1.default.Spec.JSONLD) {
            return this.fs.loadFile(filePath);
        }
        const specClient = spec_client_provider_1.default.buildClient(spectype, this.fs, filePath);
        return specClient.generateGraph(filePath, false, this.lexicalInfo);
    }
    async test() {
        const filteredExamples = this.getExamples();
        if (filteredExamples.length > 0) {
            console.log(`\u001B[0m* Processing rule directory: ${this.ruleDir.path}`);
            if (this.debug) {
                const regoPath = await this.ruleDir.generateRego(validate_1.default.singleton);
                console.log('\u001B[0m    - (debug) generating OPA Rego profile code at ' + regoPath);
            }
        }
        const exampleBuilders = filteredExamples.map(f => {
            return async () => {
                let generated = '';
                try {
                    generated = await this.parseInput(f);
                }
                catch (error) {
                    this.logger.log(`  \u001B[91m\u2717 ${f}`);
                    this.logger.log('   Error parsing spec ' + error);
                    throw new Error('   Error parsing spec ' + error);
                }
                if (!this.skipValidation) {
                    const ruleset = await profile_client_1.default.parseRuleset(this.ruleDir.profile, this.fs);
                    if (!ruleset.conforms()) {
                        this.logger.log(`  \u001B[91m\u2717 linting failed for rule definition at ${this.ruleDir.profile}`);
                        throw new Error(`Linting failed for rule definition at ${this.ruleDir.profile}`);
                    }
                    const profile = yaml.parse(fs.readFileSync(ruleset.source).toString());
                    const linter = new linter_1.default();
                    const definitionResult = linter.checkRuleDefinitions(profile);
                    if (!definitionResult.success) {
                        this.logger.log(`  \u001B[91m\u2717 profile ${ruleset.location()} found a problem with the validation: ${definitionResult.message}\u001B[0m`);
                        throw new Error(`Linting failed for rule definition at ${this.ruleDir.profile}`);
                    }
                    const termErrors = linter.lintProfileTerms(profile);
                    for (const term of Object.keys(termErrors)) {
                        const error = termErrors[term];
                        const where = error.isClass ? 'class term' : 'path property';
                        if (error.score > 0.7) {
                            this.logger.log(`  \u001B[33m! unknown ${where} '${term}' did you mean '${error.term}'\u001B[0m`);
                        }
                        else {
                            this.logger.log(`  \u001B[33m! unknown ${where} '${term}'\u001B[0m`);
                        }
                    }
                }
                return validate_1.default.singleton.validate(this.ruleDir.readProfile(), generated, false).then(r => {
                    let reportData;
                    try {
                        reportData = JSON.parse(r);
                        const validatorReport = new validator_report_1.default(reportData);
                        return new rule_tester_result_1.default(generated, f, validatorReport, r);
                    }
                    catch {
                        console.error('Could not parse report from text:');
                        console.error(`\t${r.replaceAll('\n', '\n\t')}`);
                    }
                });
            };
        });
        return promises_utils_1.default.seqProms(exampleBuilders, new Array());
    }
}
exports.default = RulesetTester;
//# sourceMappingURL=rule-tester.js.map