"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs_utils_1 = require("../fs/fs-utils");
const rule_tester_1 = require("./rule-tester");
const path = require("node:path");
const diff_1 = require("./diff");
const promises_utils_1 = require("../utils/promises-utils");
const validate_1 = require("../validate");
class TestRunner {
    constructor(location, fs, debug, skip, filter, logger, renderFiles = false) {
        this.location = location;
        this.fs = fs;
        this.debug = debug;
        this.skip = skip;
        this.filter = filter;
        this.logger = logger;
        this.renderFiles = renderFiles;
    }
    run() {
        const promises = fs_utils_1.default.listRuleDirs(this.fs, this.logger, this.location).map(ruleDir => {
            return (() => {
                return this.runTest(ruleDir);
            });
        });
        return promises_utils_1.default.seqProms(promises, new Array()).then((m) => {
            return m.flatMap((testResult) => {
                return testResult;
            });
        });
    }
    async runTest(ruleDir) {
        const reportBuilders = new rule_tester_1.default(ruleDir, this.debug, false, this.filter, this.logger, this.skip, this.fs).test();
        const results = await reportBuilders;
        const nonNullResults = results.filter(r => r !== undefined);
        const exampleBuilders = nonNullResults.flatMap(reportResult => {
            return async () => {
                const diffs = this.checkTestResult(reportResult.example, ruleDir, reportResult);
                if (this.renderFiles)
                    this.fs.writeFileSync(reportResult.example + '.report.jsonld', reportResult.rawReport);
                if (this.debug) {
                    await this.renderResult(reportResult.generated, reportResult.example);
                }
                return new diff_1.TestResult(reportResult.example, diffs);
            };
        });
        return promises_utils_1.default.seqProms(exampleBuilders, new Array());
    }
    checkTestResult(dataFile, ruleset, result) {
        const positiveTest = this.isPositive(dataFile);
        const validatorReport = result.validatorReport;
        const diffs = [];
        if (!positiveTest) {
            if (validatorReport.results().length === 0) {
                diffs.push(new diff_1.Diff(ruleset.ruleName, 'Report Conforms'));
            }
            else {
                for (const validationResult of validatorReport.results()) {
                    if (validationResult.rule !== ruleset.ruleName)
                        diffs.push(new diff_1.Diff(ruleset.ruleName, validationResult.rule));
                    if (validationResult.severity !== ruleset.severityLevel)
                        diffs.push(new diff_1.Diff(ruleset.severityLevel, validationResult.severity));
                }
            }
        }
        if ((validatorReport.results().length > 0 && positiveTest)) {
            diffs.push(new diff_1.Diff('Empty report', result.rawReport));
        }
        if (diffs.length > 0) {
            this.logger.log(`  \u001B[31m\u2715 ${dataFile}`);
        }
        else {
            this.logger.log(`  \u001B[92m\u2713 ${dataFile}`);
        }
        return diffs;
    }
    isPositive(dataFile) {
        return path.basename(dataFile).startsWith('positive');
    }
    async renderResult(generated, example) {
        const normalized = await validate_1.default.singleton.normalize(generated);
        this.fs.writeFileSync(example + '.input', normalized);
        this.logger.log(`\u001B[0m    - (debug) OPA input data: ${example}.input`);
    }
}
exports.default = TestRunner;
//# sourceMappingURL=test-runner.js.map