"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const rule_dir_1 = require("../rule-dir");
const anypoint_apb_1 = require("anypoint-apb");
const spec_client_provider_1 = require("../spec/spec-client-provider");
const fs = require("node:fs");
const sanitize = require('sanitize-filename');
const path = require('node:path');
class FsUtils {
    static existConfigFile(fs, prefix = '.') {
        return fs.existsSync(path.join(prefix, this.CONFIG_FILE));
    }
    static loadConfigFile(fs, prefix = '.') {
        const profile = fs.loadFile(path.join(prefix, this.CONFIG_FILE));
        return JSON.parse(profile);
    }
    static loadDocFile(fs, prefix = '.') {
        const pathToDoc = path.join(prefix, this.DOC_FILE);
        if (fs.existsSync(pathToDoc)) {
            return fs.loadFile(pathToDoc);
        }
        return undefined;
    }
    static loadConfigFileTemplate(profileName, fs, defaultSpec, ruleNameProfile) {
        const ruleFile = ruleNameProfile ? ruleNameProfile : FsUtils.RULE_FILE_NAME;
        const configPath = path.join(this.TEMPLATES_PATH, 'config.json');
        const config = fs.loadFile(configPath);
        return config.replace(/\${name}/g, profileName).replace(/\${defaultSpec}/g, defaultSpec).replace(/\${ruleProfileName}/g, ruleFile);
    }
    static loadDocFileTemplate(profileName, fs) {
        const configPath = path.join(this.TEMPLATES_PATH, 'documentation.md');
        return fs.loadFile(configPath);
    }
    static writeConsolidatedRuleset(fs, data, prefix = '.') {
        const pathConfigFile = path.join(prefix, this.CONFIG_FILE);
        const config = JSON.parse(fs.loadFile(pathConfigFile));
        const rulesetConfig = config.main;
        const profileFile = path.join(prefix, rulesetConfig);
        return this.writeFile(fs, profileFile, data, prefix);
    }
    static existRulesDirectory(fs, prefix = '.') {
        return fs.existsSync(path.join(prefix, this.RULES_DIR));
    }
    static writeFile(fs, file, data, prefix = '.') {
        const f = file.startsWith(path.sep) ? file : path.join(prefix, file);
        fs.writeFileSync(f, data);
        return f;
    }
    static mkDir(fs, dir, prefix = '.') {
        const d = path.join(prefix, dir);
        fs.mkdirSync(d);
        return d;
    }
    static findSpecType(fs, f) {
        if (this.isFileOrDirectory(f)) {
            return spec_client_provider_1.default.Spec.PROJECT;
        }
        if (f.endsWith('.graphql') || f.endsWith('.gql')) {
            return spec_client_provider_1.default.Spec.GRAPHQL;
        }
        if (f.endsWith('.gcl.yaml')) {
            return spec_client_provider_1.default.Spec.GCL;
        }
        if (f.endsWith('.data.jsonld')) {
            return spec_client_provider_1.default.Spec.JSONLD;
        }
        if (f.endsWith('.proto')) {
            return spec_client_provider_1.default.Spec.GRPC;
        }
        const data = fs.loadFile(f);
        if (data.includes('openapi:')) {
            return spec_client_provider_1.default.Spec.OAS;
        }
        if (data.includes('swagger:')) {
            return spec_client_provider_1.default.Spec.SWAGGER;
        }
        if (data.includes('#%RAML 1.0')) {
            return spec_client_provider_1.default.Spec.RAML;
        }
        if (data.includes('asyncapi:')) {
            return spec_client_provider_1.default.Spec.ASYNCAPI;
        }
        if (data.includes('project')) {
            return spec_client_provider_1.default.Spec.PROJECT;
        }
        throw new Error(`Cannot detect API type for file ${f}`);
    }
    static isFileOrDirectory(path) {
        const stat = fs.statSync(path);
        return stat.isDirectory();
    }
    static mkRuleDir(fs, ruleName, prefix = '.') {
        const rulesDir = path.join(prefix, this.RULES_DIR, sanitize(ruleName));
        if (!this.existRuleDir(fs, ruleName, prefix)) {
            this.mkDir(fs, path.join(this.RULES_DIR, sanitize(ruleName)), prefix);
        }
        return rulesDir;
    }
    static existRuleDir(fs, ruleName, prefix = '.') {
        const rulesDir = path.join(prefix, this.RULES_DIR, sanitize(ruleName));
        return fs.existsSync(rulesDir);
    }
    static writeProfile(fs, ruleName, data, config, prefix = '.') {
        const ruleNameProfile = config.ruleProfileName;
        const profileFile = path.join(prefix, this.RULES_DIR, sanitize(ruleName), ruleNameProfile);
        this.writeFile(fs, profileFile, data);
        return profileFile;
    }
    static loadFileNameRule(fs, prefix) {
        const config = JSON.parse(fs.loadFile(path.join(prefix, FsUtils.CONFIG_FILE)));
        if (config.ruleProfileName === undefined) {
            config.ruleProfileName = 'profile.yaml';
            return config.ruleProfileName;
        }
        return config.ruleProfileName;
    }
    static listRuleProfiles(fs, prefix = '.') {
        const acc = new Array();
        const files = fs.listDirectory(path.join(prefix, this.RULES_DIR));
        const ruleNameProfile = this.getProfileName(fs, prefix);
        for (const file of files) {
            if (this.isSameFilename(file, ruleNameProfile)) {
                acc.push(file);
            }
        }
        return acc;
    }
    static isSameFilename(file, ruleNameProfile) {
        return path.parse(file).base === ruleNameProfile;
    }
    static getProfileName(fs, prefix) {
        const config = JSON.parse(fs.loadFile(path.join(prefix, FsUtils.CONFIG_FILE)));
        return config.ruleProfileName ? config.ruleProfileName : FsUtils.PROFILE_FILE;
    }
    static listRuleDirs(fs, logger, prefix = '.') {
        const ruleNameProfile = this.loadFileNameRule(fs, prefix);
        const rulesDir = path.join(prefix, this.RULES_DIR);
        const acc = new Array();
        for (const f of fs.readdirSync(rulesDir)) {
            const nextRuleDir = path.join(rulesDir, f);
            if (fs.statSync(nextRuleDir).isDirectory()) {
                acc.push(new rule_dir_1.default(nextRuleDir, fs, logger, ruleNameProfile, rulesDir));
            }
        }
        return acc;
    }
    static exampleFolder(ruleName, positive, id, prefix) {
        // eslint-disable-next-line @typescript-eslint/ban-ts-comment
        // @ts-ignore
        const folderProject = positive ? `positive-${id}` : `negative-${id}`;
        return path.join(prefix, this.RULES_DIR, ruleName, folderProject);
    }
    static createExampleFolder(fs, ruleName, positive, id, prefix) {
        const folder = this.exampleFolder(ruleName, positive, id, prefix);
        console.log('creating folder', folder);
        fs.mkdirSync(folder);
        return folder;
    }
    static exampleFile(ruleName, format, positive, id, prefix) {
        const formatFragment = (format === this.FORMATS.GRAPHQL || format === this.FORMATS.JSONLD) ? '' : `.${format}`;
        const idFragment = id ? `-${id}` : '';
        const conformanceFragment = positive ? 'positive' : 'negative';
        // eslint-disable-next-line @typescript-eslint/ban-ts-comment
        // @ts-ignore
        const extension = this.SERIALIZATIONS[format];
        const file = `${conformanceFragment}${idFragment}${formatFragment}.${extension}`;
        return (ruleName) ? path.join(prefix, this.RULES_DIR, sanitize(ruleName), file) : path.join(prefix, file);
    }
    static existExampleData(fs, ruleName, format, positive, id, prefix) {
        return fs.existsSync(this.exampleFile(ruleName, format, positive, id, prefix));
    }
    static existFolders(fs, ruleName, positive, id, prefix) {
        return fs.existsSync(this.exampleFolder(ruleName, positive, id, prefix));
    }
    static writeExampleData(fs, ruleName, data, format, positive, id, prefix = '.') {
        const file = this.exampleFile(ruleName, format, positive, id, prefix);
        fs.writeFileSync(file, data);
        return file;
    }
    static loadProfileTemplate(fs, profileName) {
        const profilePath = path.join(this.TEMPLATES_PATH, 'default_profile.yaml');
        const profile = fs.loadFile(profilePath);
        // eslint-disable-next-line no-template-curly-in-string
        return profile.replace('${name}', profileName);
    }
    static loadDefaultSpec(fs, format) {
        if (Object.values(this.FORMATS).includes(format)) {
            if (format === 'project') {
                return path.join(this.TEMPLATES_PATH, FsUtils.FORMATS.OAS);
            }
            // eslint-disable-next-line @typescript-eslint/ban-ts-comment
            // @ts-ignore
            const file = path.join(this.TEMPLATES_PATH, `default_${format}.${this.SERIALIZATIONS[format]}`);
            return fs.loadFile(file);
        }
        throw new Error(`Unknown API format ${format}, cannot find default API template`);
    }
    static readRuleTemplate(spec, fs) {
        const source = path.join(FsUtils.TEMPLATES_PATH, 'new', 'rule', spec, 'rule.yaml');
        return fs.readFileSync(source);
    }
    static copyRuleExamplesForSpec(spec, target, fs) {
        const source = path.join(FsUtils.TEMPLATES_PATH, 'new', 'rule', spec);
        // eslint-disable-next-line unicorn/consistent-function-scoping
        const everythingButRule = (source, _) => !source.includes('rule.yaml');
        fs.cpSync(source, target, { recursive: true, filter: everythingButRule });
    }
    static loadExchangeJson(fs, defaultSpec = this.FORMATS.OAS) {
        // eslint-disable-next-line @typescript-eslint/ban-ts-comment
        // @ts-ignore
        const exchangeJsonpath = path.join(this.TEMPLATES_PATH, FsUtils.EXCHANGE_JSON);
        const exchangeJson = fs.loadFile(exchangeJsonpath);
        // eslint-disable-next-line no-template-curly-in-string
        return exchangeJson.replace('${defaultSpec}', defaultSpec);
    }
    static fsToResourceLoader(fs, base, error = this.logNothing) {
        const loader = new FsResourceLoader(fs, base, error);
        return anypoint_apb_1.ResourceLoaderFactory.create(loader);
    }
}
exports.default = FsUtils;
FsUtils.TEMPLATES_PATH = path.join(__dirname, '..', '..', 'templates');
FsUtils.ROOT_PATH = path.join(__dirname, '..', '..', '..');
FsUtils.CONFIG_FILE = 'config.json';
FsUtils.RULE_FILE_NAME = 'rule';
FsUtils.DOC_FILE = 'documentation.md';
FsUtils.RULES_DIR = 'rules';
FsUtils.PROFILE_FILE = 'profile.yaml';
FsUtils.EXCHANGE_JSON = 'default_exchange.json';
FsUtils.RULESET_DIALECT = require.resolve('@aml-org/amf-validation-profile-dialect/validation-profile.yaml');
FsUtils.FORMATS = {
    OAS: 'oas',
    SWAGGER: 'swagger',
    RAML: 'raml',
    ASYNCAPI: 'asyncapi',
    GRAPHQL: 'graphql',
    GRPC: 'grpc',
    JSONLD: 'jsonld',
    GCL: 'gcl',
    PROJECT: 'project',
};
FsUtils.SERIALIZATIONS = {
    oas: 'yaml',
    swagger: 'yaml',
    raml: 'yaml',
    asyncapi: 'yaml',
    graphql: 'graphql',
    grpc: 'proto',
    jsonld: 'data.jsonld',
    gcl: 'yaml',
    project: 'project',
};
// eslint-disable-next-line @typescript-eslint/no-empty-function
FsUtils.logNothing = (_) => { };
const tryOpenFileSync = (fs, resource, error) => {
    try {
        const value = fs.loadFile(resource);
        return Promise.resolve(value);
    }
    catch (error_) {
        error(`Native error: ${error_}`);
        return Promise.reject(error_);
    }
};
class FsResourceLoader {
    // This constructor isn't useless as it sets private properties
    // eslint-disable-next-line no-useless-constructor
    constructor(fs, base, error) {
        this.fs = fs;
        this.base = base;
        this.error = error;
    }
    accepts(resource) {
        const path = this.preparePath(resource);
        const fileExists = this.fs.existsSync(path);
        if (!fileExists) {
            this.error(`File ${resource} doesn't seem to exist`);
        }
        return fileExists;
    }
    stripFileProtocol(resource) {
        return resource.replace('file://', '');
    }
    prependBase(resource) {
        if (this.base !== undefined && !resource.startsWith(this.base)) {
            return path.resolve(this.base, resource);
        }
        return resource;
    }
    async fetch(resource) {
        const uriWithScheme = this.preparePath(resource);
        const content = await tryOpenFileSync(this.fs, uriWithScheme, this.error);
        return Promise.resolve(new anypoint_apb_1.Content(content, resource));
    }
    preparePath(resource) {
        return this.prependBase(this.stripFileProtocol(resource));
    }
}
//# sourceMappingURL=fs-utils.js.map