"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AmfModel = void 0;
const yaml = require("yaml");
const path = require("node:path");
const fs = require("node:fs");
const collection_utils_1 = require("../utils/collection-utils");
const ss = require('string-similarity');
/**
 * Processes the mode.yaml file generated from the AMF model
 * and provides methods to query the classes and terms in the AMF model.
 */
class AmfModel {
    constructor(profilePrefixes) {
        this.CONTEXT = {
            'http://a.ml/vocabularies/data#': 'data',
            'http://www.w3.org/ns/shacl#': 'shacl',
            'http://a.ml/vocabularies/shapes#': 'shapes',
            'http://a.ml/vocabularies/document#': 'doc',
            'http://a.ml/vocabularies/meta#': 'meta',
            'http://a.ml/vocabularies/apiContract#': 'apiContract',
            'http://a.ml/vocabularies/core#': 'core',
            'http://www.w3.org/2001/XMLSchema#': 'xsd',
            'http://www.w3.org/2000/01/rdf-schema': 'rdfs',
            'http://www.w3.org/1999/02/22-rdf-syntax-ns#': 'rdf',
            'http://a.ml/vocabularies/security#': 'security',
            'http://a.ml/vocabularies/document-source-maps#': 'sourcemaps',
            'http://a.ml/vocabularies/api-extension#': 'apiExt',
        };
        this._invertedContext = undefined;
        this.VALIDATOR_PREFIXES = {
            apiExt: 'http://a.ml/vocabularies/api-extension#',
        };
        this.classList = [];
        this.propertyList = [];
        const modelContent = AmfModel.MODEL_PATH.map(modelPath => {
            return yaml.parse(fs.readFileSync(modelPath).toString());
        });
        this.model = collection_utils_1.default.flatValues(modelContent);
        this.buildTypesList();
        this.buildFieldList();
        this.profilePrefixes = profilePrefixes;
    }
    get invertedContext() {
        if (this._invertedContext) {
            return this._invertedContext;
        }
        this._invertedContext = {};
        // eslint-disable-next-line guard-for-in
        for (const p in this.CONTEXT) {
            const v = this.CONTEXT[p];
            this._invertedContext[v] = p;
        }
        return this._invertedContext;
    }
    findMatch(term, model, isClass) {
        let bestMatch = '';
        let bestScore = 0;
        for (const c of Object.keys(this.profilePrefixes)) {
            if (term.startsWith(c)) {
                return {
                    source: term,
                    term: term,
                    score: 1,
                    match: true,
                    isClass: isClass,
                    isProperty: !isClass,
                    isRegex: false,
                };
            }
        }
        for (const c of Object.keys(this.VALIDATOR_PREFIXES)) {
            if (term.startsWith(c)) {
                return {
                    source: term,
                    term: term,
                    score: 1,
                    match: true,
                    isClass: isClass,
                    isProperty: !isClass,
                    isRegex: false,
                };
            }
        }
        for (const c of model) {
            const score = ss.compareTwoStrings(term, c);
            if (score === 1) {
                return {
                    source: term,
                    term: c,
                    score: 1,
                    match: true,
                    isClass: isClass,
                    isProperty: !isClass,
                    isRegex: false,
                };
            }
            if (score > bestScore) {
                bestMatch = c;
                bestScore = score;
            }
        }
        return {
            source: term,
            term: bestMatch,
            score: bestScore,
            match: false,
            isClass: isClass,
            isProperty: !isClass,
            isRegex: false,
        };
    }
    findClassMatch(term) {
        return this.findMatch(term, this.classList, true);
    }
    findPropertyMatch(term) {
        const parts = term
            .replace(/ /g, '')
            .replace(/\^/g, '')
            .replace(/\(/g, '')
            .replace(/\)/g, '')
            .replace(/\//g, ' ')
            .replace(/\|/g, ' ')
            .split(/ /g);
        return parts.map(p => this.findMatch(p, this.propertyList, false));
    }
    buildTypesList() {
        const acc = {};
        for (const nodeType of this.model) {
            const tls = (nodeType.types);
            // eslint-disable-next-line no-eq-null,eqeqeq
            if (tls != null) {
                for (const className of tls) {
                    const compacted = this.compact(className);
                    acc[compacted] = true;
                    acc[compacted] = true;
                }
            }
        }
        this.classList = Object.keys(acc);
    }
    buildFieldList() {
        const acc = {};
        for (const nodeType of this.model) {
            const fields = nodeType.fields;
            // eslint-disable-next-line no-eq-null,eqeqeq
            if (fields != null) {
                // eslint-disable-next-line unicorn/no-array-for-each
                fields.forEach((field) => {
                    const compacted = this.compact(field.type);
                    acc[compacted] = true;
                });
            }
        }
        this.propertyList = Object.keys(acc);
    }
    expand(curie) {
        const parts = curie.includes('.') ? curie.split('.') : curie.split(':');
        const namespace = this.invertedContext[parts[0]];
        return namespace ? `${namespace}:${parts[1]}` : curie;
    }
    compact(className) {
        for (const k in this.CONTEXT) {
            if (className.startsWith(k)) {
                const prefix = this.CONTEXT[k];
                return `${prefix}.${className.replace(k, '')}`;
            }
        }
        return className;
    }
    compactPath(path) {
        let acc = path;
        for (const k in this.CONTEXT) {
            if (acc.includes(k)) {
                while (acc.includes(k)) {
                    acc = acc.replace(k, this.CONTEXT[k] + '.');
                }
            }
        }
        return acc;
    }
}
exports.AmfModel = AmfModel;
AmfModel.MODEL_PATH = [
    path.join(__dirname, '..', 'model.yaml'),
    path.join(__dirname, '..', 'project-model.yaml'),
];
//# sourceMappingURL=amf-model.js.map