"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.IndexEntry = exports.AMFLexicalIndex = void 0;
const position_1 = require("./position");
const range_1 = require("./range");
const generate_1 = require("../../spec/api-contract/generate");
const amf_model_1 = require("../amf-model");
const PropUtils = {
    cleanProperty(s) {
        return s.replace(':', '.').replace('raml-', '');
    },
};
class AMFLexicalIndex {
    constructor(api) {
        this.index = [];
        this.amfModel = new amf_model_1.AmfModel({});
        this.api = api;
        // eslint-disable-next-line prefer-regex-literals
        this.lexicalRgx = new RegExp('\\[\\((\\d+),(\\d+)\\)-\\((\\d+),(\\d+)\\)]');
    }
    async initialize() {
        const rendered = await generate_1.default.generate(this.api, false, true);
        const jsonld = JSON.parse(rendered);
        const lexicalIdx = this.indexLexicalsById(jsonld);
        this.indexNodesByLexicalRange(jsonld, lexicalIdx);
    }
    rootPath(to) {
        const acc = [];
        let current = to;
        // eslint-disable-next-line no-constant-condition
        while (true) {
            const parent = this.index.find(e => {
                return e.getLinkTargets().includes(current.id);
            });
            if (!parent) {
                return acc.reverse().map(p => PropUtils.cleanProperty(p));
            }
            // eslint-disable-next-line array-callback-return
            const link = parent.getPropertyUris().map((p) => {
                const values = (parent.getValue(p) || []);
                for (const v of values) {
                    if (v['@id'] === current.id) {
                        return p;
                    }
                }
            }).find(r => r !== undefined);
            if (!link) {
                return acc.reverse().map(p => PropUtils.cleanProperty(p));
            }
            acc.push(link);
            current = parent;
        }
    }
    findById(jsonld, id) {
        return jsonld['@graph'].find((n) => n['@id'] === id);
    }
    indexNodesByLexicalRange(jsonld, lexicalIdx) {
        for (const node of jsonld['@graph']) {
            const types = node['@type'];
            const id = node['@id'];
            const lexicalInfo = lexicalIdx[id];
            if (types && types.includes('doc:DomainElement') && lexicalInfo) {
                const match = lexicalInfo.match(this.lexicalRgx);
                if (match) {
                    const c1 = match[1];
                    const l1 = match[2];
                    const c2 = match[3];
                    const l2 = match[4];
                    if (match && c1 && l1 && c2 && l2) {
                        const start = new position_1.Position(Number.parseInt(c1, 10) - 1, Number.parseInt(l1, 10));
                        const end = new position_1.Position(Number.parseInt(c2, 10) - 1, Number.parseInt(l2, 10) - 1);
                        const range = new range_1.Range(start, end);
                        const entry = new IndexEntry(range, node, this.amfModel);
                        this.index.push(entry);
                    }
                }
            }
            else if (types && types.includes('doc:DomainElement') && !lexicalInfo) {
                // let's synthesize for elements generated but without proper lexical node
                let startAcc;
                let endAcc;
                const sourceMapIdNode = node['sourcemaps:sources'][0] || {};
                const sourceMapId = sourceMapIdNode['@id'];
                const sourceMapNode = this.findById(jsonld, sourceMapId);
                const sourceMapLexicalNodes = sourceMapNode['sourcemaps:lexical'];
                if (sourceMapLexicalNodes && id && !lexicalIdx[id]) {
                    for (const lexicalInfoNode of sourceMapLexicalNodes) {
                        const lexicalId = lexicalInfoNode['@id'];
                        const lexicalInfoElementNode = this.findById(jsonld, lexicalId) || {};
                        const lexicalInfoValue = lexicalInfoElementNode['sourcemaps:value'];
                        if (lexicalInfoValue) {
                            const match = lexicalInfoValue.match(this.lexicalRgx);
                            if (match) {
                                const c1 = match[1];
                                const l1 = match[2];
                                const c2 = match[3];
                                const l2 = match[4];
                                if (match && c1 && l1 && c2 && l2) {
                                    const start = new position_1.Position(Number.parseInt(c1, 10) - 1, Number.parseInt(l1, 10));
                                    const end = new position_1.Position(Number.parseInt(c2, 10) - 1, Number.parseInt(l2, 10) - 1);
                                    if (!startAcc || startAcc.compareTo(start) === -1) {
                                        startAcc = start;
                                    }
                                    if (!endAcc || endAcc.compareTo(end) === 1) {
                                        endAcc = end;
                                    }
                                }
                            }
                        }
                    }
                }
                if (startAcc && endAcc) {
                    const range = new range_1.Range(startAcc, endAcc);
                    const entry = new IndexEntry(range, node, this.amfModel);
                    this.index.push(entry);
                }
            }
            else {
                this.index.push(new IndexEntry(new range_1.Range(new position_1.Position(0, 0), new position_1.Position(0, 0)), node, this.amfModel));
            }
        }
    }
    indexLexicalsById(jsonld) {
        const base = jsonld['@context']['@base'];
        const lexicalIdx = {};
        for (const node of jsonld['@graph']) {
            if (node['sourcemaps:element']) {
                const id = node['sourcemaps:element'].replace(base, '');
                const range = node['sourcemaps:value'];
                if (id && range && this.lexicalRgx.test(range)) {
                    lexicalIdx[id] = range;
                }
            }
        }
        return lexicalIdx;
    }
    entryFor(position) {
        let current = [];
        for (const entry of this.index) {
            const inRange = entry.range.containsPosition(position);
            if (current.length === 0 && inRange) {
                current = [entry];
            }
            else if (current && inRange && entry.range.compareTo(current[0].range) === 0) {
                current.push(entry);
            }
            else if (current && inRange && entry.isMoreSpecificThan(current[0])) {
                current = [entry];
            }
            else {
                // skip
            }
        }
        return current;
    }
}
exports.AMFLexicalIndex = AMFLexicalIndex;
class IndexEntry {
    constructor(range, object, amfModel) {
        this._range = range;
        this.object = object;
        this.amfModel = amfModel;
    }
    get range() {
        return this._range;
    }
    get id() {
        return this.object['@id'];
    }
    isMoreSpecificThan(other) {
        return other._range.containsRange(this._range);
    }
    getClass() {
        return this.object['@type'] ? PropUtils.cleanProperty(this.object['@type'][0]) : '';
    }
    getProperties() {
        const acc = [];
        for (const p of Object.keys(this.object)) {
            // eslint-disable-next-line no-prototype-builtins
            if (this.object.hasOwnProperty(p) && !p.startsWith('sourcemaps') && !p.startsWith('@')) {
                const values = this.getValue(p);
                if (p === 'shacl:datatype') {
                    const value = values[0]['@id'];
                    acc.push(`${PropUtils.cleanProperty(p)}: ${this.amfModel.expand(value)}`);
                }
                if (values.length > 0 && values[0] && !values[0]['@id']) {
                    const value = values.map(v => `${v}`).join(',');
                    acc.push(`${PropUtils.cleanProperty(p)}: ${value}`);
                }
            }
        }
        return acc;
    }
    getPropertyUris() {
        return Object.keys(this.object);
    }
    getValue(property) {
        const tmp = this.object[property];
        if (tmp) {
            return tmp.constructor.name === 'Array' ? tmp : [tmp];
        }
        return [];
    }
    getLinkTargets() {
        return Object.values(this.object).flatMap((n) => {
            return n.constructor.name === 'Array' ? n.map((nested) => nested['@id']) : [n['@id']];
        }).filter((n) => n !== undefined);
    }
}
exports.IndexEntry = IndexEntry;
//# sourceMappingURL=amf-lexical-index.js.map