"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EditorPrompt = void 0;
const chalk = require('chalk');
const Base = require('inquirer/lib/prompts/base');
const observe = require('inquirer/lib/utils/events');
// eslint-disable-next-line node/no-extraneous-require
const { filter, share } = require('rxjs/operators');
const position_1 = require("./position");
// eslint-disable-next-line node/no-extraneous-require
const shellCursor = require('cli-cursor');
class EditorPrompt extends Base {
    constructor(questions, rl, answers) {
        super(questions, rl, answers);
        this.showProperties = false;
        this.showPath = false;
        this.showIds = false;
        this.line = 0;
        this.column = 0;
        this.lines = this.opt.content.split('\n');
        this.AMFLexicalIndex = this.opt.index;
        const events = observe(this.rl);
        shellCursor.hide();
        // eslint-disable-next-line unicorn/no-array-for-each
        events.normalizedUpKey.forEach(this.onUpKey.bind(this));
        // eslint-disable-next-line unicorn/no-array-for-each
        events.normalizedDownKey.forEach(this.onDownKey.bind(this));
        events.keypress
            .pipe(filter((evt) => evt.key.name === 'left'), share())
            // eslint-disable-next-line unicorn/no-array-for-each
            .forEach(this.onLeftKey.bind(this));
        events.keypress
            .pipe(filter((evt) => evt.key.name === 'right'), share())
            // eslint-disable-next-line unicorn/no-array-for-each
            .forEach(this.onRightKey.bind(this));
        events.keypress
            .pipe(filter((evt) => evt.key.name === 'p'), share())
            // eslint-disable-next-line unicorn/no-array-for-each
            .forEach(this.onPKey.bind(this));
        events.keypress
            .pipe(filter((evt) => evt.key.name === 'l'), share())
            // eslint-disable-next-line unicorn/no-array-for-each
            .forEach(this.onLKey.bind(this));
        events.keypress
            .pipe(filter((evt) => evt.key.name === 'i'), share())
            // eslint-disable-next-line unicorn/no-array-for-each
            .forEach(this.onIKey.bind(this));
    }
    lastColumn() {
        return this.lines[this.line].length - 1;
    }
    lastLine() {
        return this.lines.length - 1;
    }
    onUpKey() {
        this.moveCursor('up');
        this.render();
    }
    onPKey() {
        this.showProperties = !this.showProperties;
        this.render();
    }
    onLKey() {
        this.showPath = !this.showPath;
        this.render();
    }
    onIKey() {
        this.showIds = !this.showIds;
        this.render();
    }
    onDownKey() {
        this.moveCursor('down');
        this.render();
    }
    onLeftKey() {
        this.moveCursor('left');
        this.render();
    }
    onRightKey() {
        this.moveCursor('right');
        this.render();
    }
    menu() {
        const space = '    ';
        return chalk.bgGray(`←↑→↓: move ${space}l: toggle link ${space}i: toggle @ids ${space}p: toggle properties ${space} ^c: quit\n`);
    }
    render() {
        let output = '';
        const currentPosition = new position_1.Position(this.line, this.column);
        const entries = this.AMFLexicalIndex.entryFor(currentPosition);
        const entry = entries[entries.length - 1];
        for (let i = 0; i < this.lines.length; i++) {
            const currentLine = this.lines[i];
            for (let j = 0; j < currentLine.length; j++) {
                const pos = new position_1.Position(i, j);
                const char = this.lines[i][j];
                if (currentPosition.compareTo(pos) === 0) {
                    output += chalk.bgGray(char); // draw cursor
                }
                else if (entry && entry.range.containsPosition(pos)) {
                    output += chalk.bgBlueBright(char); // color selected node
                }
                else {
                    output += char;
                }
            }
            output += '\n';
            if (entry && currentPosition.line === i) {
                const padding = '    ';
                for (const e of entries) {
                    const klass = e.getClass();
                    output += chalk.black.bgCyan(klass) + '\n';
                    if (this.showIds) {
                        output += chalk.white.bgRed(padding + e.id + '\n');
                    }
                    if (this.showProperties) {
                        for (const prop of e.getProperties()) {
                            output += chalk.black.bgGreenBright(padding + prop + '\n');
                        }
                    }
                    if (this.showPath) {
                        let path = '';
                        if (entry) {
                            const pathComps = this.AMFLexicalIndex.rootPath(e);
                            pathComps.shift();
                            path = pathComps.join(' / ');
                        }
                        output += chalk.white.bgMagenta(padding + path + '\n');
                    }
                }
            }
        }
        output += this.menu();
        this.screen.render(output);
    }
    _run() {
        this.render();
    }
    moveCursor(movement) {
        // eslint-disable-next-line unicorn/consistent-function-scoping
        const clip = (i, min, max) => {
            if (i < min)
                return min;
            if (i > max)
                return max;
            return i;
        };
        switch (movement) {
            case 'up':
                this.line = clip(this.line - 1, 0, this.lastLine());
                this.column = clip(this.column, 0, this.lastColumn());
                break;
            case 'down':
                this.line = clip(this.line + 1, 0, this.lastLine());
                this.column = clip(this.column, 0, this.lastColumn());
                break;
            case 'left':
                if (this.column === 0 && this.line > 0) {
                    // move up a line
                    this.line = clip(this.line - 1, 0, this.lastLine());
                    this.column = this.lastColumn();
                }
                else {
                    this.column = clip(this.column - 1, 0, this.lastColumn());
                }
                break;
            case 'right':
                if (this.column === this.lastColumn() && this.line < this.lastLine()) {
                    // move down a line
                    this.line = clip(this.line + 1, 0, this.lastLine());
                    this.column = 0;
                }
                else {
                    this.column = clip(this.column + 1, 0, this.lastColumn());
                }
        }
    }
}
exports.EditorPrompt = EditorPrompt;
//# sourceMappingURL=editor.js.map