"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ProofBuilder = void 0;
const validator_report_1 = require("./validator-report");
const proof_1 = require("./traces/proof");
const node_1 = require("./traces/node");
const property_1 = require("./traces/property");
const constraint_1 = require("./traces/constraint");
const link_1 = require("./traces/link");
class ProofBuilder {
    constructor(result, model, normalized) {
        this.report = result;
        this.model = model;
        this.normalized = JSON.parse(normalized);
    }
    normalizedNode(id) {
        return this.normalized['@ids'][id];
    }
    nodeInfo(id) {
        const node = this.normalizedNode(id);
        if (node) {
            const types = node['@type'] || [];
            const nodeType = this.model.compact(types[0]);
            const nodeId = this.idForNode(node, nodeType);
            return {
                name: nodeId,
                type: nodeType,
            };
        }
    }
    build(compact) {
        const results = this.report.results();
        const proofs = results.map(error => {
            const proof = this.buildProof(error);
            if (compact)
                proof.compact();
            return proof;
        });
        const acc = {};
        let aggregatedProofs = proofs;
        if (compact) {
            for (const p of proofs) {
                const present = acc[p.validation];
                if (present) {
                    present.merge(p);
                    present.compact();
                }
                else {
                    acc[p.validation] = p;
                }
            }
            aggregatedProofs = Object.values(acc);
        }
        return aggregatedProofs;
    }
    buildProof(error) {
        const proof = new proof_1.Proof(error.rule, error.severity, error.message, error.location);
        proof.nodes.push(this.buildNode(error));
        return proof;
    }
    buildNode(error) {
        const nodeInfo = this.nodeInfo(error.focusNode);
        if (nodeInfo) {
            const node = new node_1.Node(nodeInfo.name, nodeInfo.type, error.location);
            for (const t of error.trace)
                this.buildTrace(node, t);
            return node;
        }
        throw new Error('Cannot find focusNode ' + error.focusNode);
    }
    buildTrace(node, traceData) {
        (traceData.component === 'nested' || traceData.component === 'atLeast' || traceData.component === 'atMost') ? this.nestNode(node, traceData) : this.collectProperties(node, traceData);
    }
    idForNode(node, nodeType) {
        let name;
        const id = node['@id'];
        switch (nodeType) {
            case 'apiContract.WebAPI': {
                name = node['http://a.ml/vocabularies/core#name'];
                break;
            }
            case 'apiContract.EndPoint': {
                name = node['http://a.ml/vocabularies/apiContract#path'];
                break;
            }
            case 'apiContract.Operation': {
                name = node['http://a.ml/vocabularies/apiContract#operationId'] || node['http://a.ml/vocabularies/apiContract#guiSummary'] || node['http://a.ml/vocabularies/core#name'] || node['http://a.ml/vocabularies/apiContract#method'];
                break;
            }
            case 'apiContract.Response': {
                name = node['http://a.ml/vocabularies/core#name'] || node['http://a.ml/vocabularies/apiContract#statusCode'];
                break;
            }
            case 'apiContract.Request': {
                name = node['http://a.ml/vocabularies/core#name'] || node['http://a.ml/vocabularies/apiContract#statusCode'];
                break;
            }
            case 'apiContract.Parameter': {
                name = node['http://a.ml/vocabularies/core#name'];
                break;
            }
            default: {
                name = node['http://www.w3.org/ns/shacl#name'] || node['http://a.ml/vocabularies/core#name'];
                break;
            }
        }
        if (name) {
            return `'${name}' (@id: ${id})`;
        }
        return `(@id: ${id})`;
    }
    nestNode(node, traceData) {
        const component = traceData.component;
        const path = this.model.compactPath(traceData.resultPath);
        const successfulNodes = traceData.traceValue.successfulNodes;
        const failedNodes = traceData.traceValue.failedNodes;
        const cardinality = traceData.traceValue.cardinality;
        const results = traceData.traceValue.subResult.map(r => new validator_report_1.ValidatorResult(r));
        const negated = traceData.traceValue.negated;
        const link = new link_1.Link(path, successfulNodes, failedNodes, cardinality, component, negated, traceData.location);
        for (const r of results) {
            link.nodes.push(this.buildNode(r));
        }
        node.links.push(link);
    }
    collectProperties(node, traceData) {
        const path = this.model.compactPath(traceData.resultPath);
        if (traceData.component === 'datatype') {
            traceData.traceValue.expected = this.model.compactPath(traceData.traceValue.expected);
        }
        const constraint = new constraint_1.Constraint(traceData.component, traceData.traceValue);
        const property = new property_1.Property(path, traceData.location);
        property.constraints.push(constraint);
        node.properties.push(property);
    }
}
exports.ProofBuilder = ProofBuilder;
//# sourceMappingURL=proof-builder.js.map