"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("node:fs");
const validate_1 = require("./validate");
const path = require("node:path");
const yaml = require("yaml");
const amf_model_1 = require("./model/amf-model");
const fs_utils_1 = require("./fs/fs-utils");
const spec_client_provider_1 = require("./spec/spec-client-provider");
const profile_client_1 = require("./spec/aml/profile-client");
const linter_1 = require("./commands/linter");
class RuleDir {
    constructor(path, fs, logger, ruleProfileName, directory) {
        this._negatedRule = false;
        this._profilePath = '';
        this.logger = logger;
        this._path = path;
        this._positive = [];
        this._negative = [];
        this._fs = fs;
        this.directory = directory;
        const contents = fs.listDirectory(path);
        let profileNameProfile = false;
        for (const c of contents) {
            if (c.includes(ruleProfileName)) {
                this._profilePath = c;
            }
            else if (c.includes('profile.yaml')) {
                profileNameProfile = true;
            }
            else if (c.includes('positive') && c.endsWith('data.jsonld')) {
                this._positive.push(c);
            }
            else if (c.includes('negative') && c.endsWith('data.jsonld')) {
                this._negative.push(c);
            }
            else if (c.includes('positive') && !c.includes('jsonld') && !c.endsWith('.input')) {
                this._positive.push(c);
            }
            else if (c.includes('negative') && !c.includes('jsonld') && !c.endsWith('.input')) {
                this._negative.push(c);
            }
        }
        if (this._profilePath === '') {
            let message = 'Couldnt find ' + ruleProfileName;
            if (profileNameProfile) {
                message += ' but found a profile.yaml please re-name the file';
            }
            this.logger.error(message);
        }
    }
    get parsedRuleset() {
        // eslint-disable-next-line no-eq-null,eqeqeq
        if (this._parsed == null) {
            this._parsed = yaml.parse(this.readProfile());
        }
        return this._parsed;
    }
    negateRule() {
        if (!this._negatedRule) {
            this._negatedRule = true;
            const rule = this.ruleBody;
            const newRule = {};
            for (const p in rule) {
                if (p === 'message' || p === 'targetClass') {
                    newRule[p] = rule[p];
                    delete rule[p];
                }
            }
            newRule.not = rule;
            this.parsedRuleset.validations[this.ruleName] = newRule;
        }
    }
    get ruleName() {
        const validations = this.parsedRuleset.validations;
        return Object.keys(validations)[0];
    }
    get tags() {
        return this.parsedRuleset.tags;
    }
    get prefixes() {
        return this.parsedRuleset.prefixes;
    }
    get ruleBody() {
        const validations = this.parsedRuleset.validations;
        return Object.values(validations)[0];
    }
    get severityLevel() {
        const rule = this.ruleName;
        // eslint-disable-next-line no-eq-null,eqeqeq
        if (this.parsedRuleset.warning != null && this.parsedRuleset.warning.includes(rule)) {
            return 'warning';
        }
        // eslint-disable-next-line no-eq-null,eqeqeq
        if (this.parsedRuleset.violation != null && this.parsedRuleset.violation.includes(rule)) {
            return 'violation';
        }
        // eslint-disable-next-line no-eq-null,eqeqeq
        if (this.parsedRuleset.info != null && this.parsedRuleset.info.includes(rule)) {
            return 'info';
        }
        throw new Error('cannot find the severity level for ruleset ' + this.path);
    }
    get negative() {
        return this._negative;
    }
    get positive() {
        return this._positive;
    }
    findExample(id) {
        const allExamples = [...this.negative, ...this.positive];
        return allExamples.find(example => {
            return example.includes(id);
        });
    }
    enrichedAmfModel() {
        const profile = yaml.parse(this.readProfile());
        return new amf_model_1.AmfModel(profile.prefixes || {});
    }
    async exampleToJSONLD(id, sourceMaps) {
        const f = this.findExample(id);
        if (f) {
            const spectype = fs_utils_1.default.findSpecType(this._fs, f);
            if (spectype === spec_client_provider_1.default.Spec.JSONLD) {
                return this._fs.loadFile(f);
            }
            const specClient = spec_client_provider_1.default.buildClient(spectype, this._fs, f);
            const generated = specClient.generateGraph(f, false, sourceMaps);
            return generated;
        }
        throw new Error(`Cannot find example for id ${id}`);
    }
    async validateRuleset() {
        return profile_client_1.default.parseRuleset(this.profile, this._fs);
    }
    async validate(source, skipValidation, debug, sourceMaps) {
        const log = [];
        let generated;
        try {
            generated = await this.exampleToJSONLD(source, sourceMaps);
        }
        catch (error) {
            log.push(`  \u001B[91m\u2717 ${source}`, '   Error parsing spec ' + error);
            return {
                success: false,
                log: log,
            };
        }
        if (!skipValidation) {
            const rulesetAmfReport = await this.validateRuleset();
            if (!rulesetAmfReport.conforms()) {
                log.push(`  \u001B[91m\u2717 linting failed for rule definition at ${this.profile}`);
                return {
                    success: false,
                    generated: generated,
                    log: log,
                };
            }
            const linter = new linter_1.default();
            const termErrors = linter.lintProfileTerms(rulesetAmfReport.source);
            for (const term of Object.keys(termErrors)) {
                const error = termErrors[term];
                const where = error.isClass ? 'class term' : 'path property';
                if (error.score > 0.7) {
                    log.push(`  \u001B[33m! unknown ${where} '${term}' did you mean '${error.term}'\u001B[0m`);
                }
                else {
                    log.push(`  \u001B[33m! unknown ${where} '${term}'\u001B[0m`);
                }
            }
        }
        let normalized;
        const report = await validate_1.default.singleton.validate(this.readProfile(), generated, false);
        if (debug) {
            normalized = await validate_1.default.singleton.normalize(generated);
        }
        return {
            success: true,
            log: log,
            reportJSON: report,
            generated: generated,
            normalized: normalized,
        };
    }
    get profile() {
        return this._profilePath;
    }
    get path() {
        return this._path;
    }
    readProfile() {
        return this._parsed ? yaml.stringify(this._parsed) : fs.readFileSync(this._profilePath).toString();
    }
    async generateRego(validator) {
        const profile = this.readProfile();
        const rego = await validator.generateRego(profile);
        const regoPath = path.join(this._path, 'profile.rego');
        fs.writeFileSync(regoPath, rego);
        return regoPath;
    }
}
exports.default = RuleDir;
//# sourceMappingURL=rule-dir.js.map