"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Constraint = void 0;
const colors_1 = require("../colors");
class Constraint {
    constructor(component, traceValue) {
        this.component = component;
        this.traceValue = traceValue;
    }
    get invocation() {
        let invocation;
        switch (this.component) {
            case 'pattern': {
                invocation = `pattern(/${this.traceValue.expected || ''}/)`;
                break;
            }
            case 'lessThan': {
                invocation = `lessThan('${this.traceValue.altPath}')`;
                break;
            }
            case 'lessThanOrEquals': {
                invocation = `lessThanOrEquals('${this.traceValue.altPath}')`;
                break;
            }
            case 'equalsTo': {
                invocation = `equalsTo('${this.traceValue.altPath}')`;
                break;
            }
            case 'disjointWith': {
                invocation = `disjointWith('${this.traceValue.altPath}')`;
                break;
            }
            case 'rego': {
                invocation = 'rego()';
                break;
            }
            case 'datatype': {
                invocation = `datatype("${this.traceValue.expected}")`;
                break;
            }
            default: {
                invocation = `${this.component}(${this.traceValue.expected})`;
                break;
            }
        }
        if (this.traceValue.negated) {
            invocation = `not ${invocation}`;
        }
        return invocation;
    }
    // eslint-disable-next-line complexity
    get explanation() {
        let explanation;
        switch (this.component) {
            case 'pattern': {
                if (this.traceValue.negated) {
                    explanation = `pattern matches "${this.traceValue.actual || this.traceValue.argument}"`;
                }
                else {
                    explanation = `pattern does not match "${this.traceValue.actual || this.traceValue.argument}"`;
                }
                break;
            }
            case 'minCount': {
                if (this.traceValue.expected === 1) {
                    explanation = this.traceValue.negated ? 'found a value when property should have none' : 'missing mandatory property';
                }
                else if (this.traceValue.negated) {
                    explanation = `${this.traceValue.actual} values found are more or equal than the at most ${this.traceValue.expected} expected ones`;
                }
                else {
                    explanation = `${this.traceValue.actual} values found are less than the at least ${this.traceValue.expected} expected ones`;
                }
                break;
            }
            case 'maxCount': {
                if (this.traceValue.negated) {
                    explanation = `${this.traceValue.actual} values found are less than or equal than the at least ${this.traceValue.expected} expected ones`;
                }
                else {
                    explanation = `${this.traceValue.actual} values found are more than the at most ${this.traceValue.expected} expected ones`;
                }
                break;
            }
            case 'exactCount': {
                if (this.traceValue.negated) {
                    explanation = `${this.traceValue.actual} values found should not match ${this.traceValue.expected} values`;
                }
                else {
                    explanation = `${this.traceValue.actual} values found don't match the ${this.traceValue.expected} expected ones`;
                }
                break;
            }
            case 'maximumExclusive': {
                if (this.traceValue.negated) {
                    explanation = `${this.traceValue.actual} string length found smaller than the expected ${this.traceValue.expected} length`;
                }
                else {
                    explanation = `${this.traceValue.actual} string length found larger or equal than the expected ${this.traceValue.expected} length`;
                }
                break;
            }
            case 'maximumInclusive': {
                if (this.traceValue.negated) {
                    explanation = `${this.traceValue.actual} string length found smaller or equal than the expected ${this.traceValue.expected} length`;
                }
                else {
                    explanation = `${this.traceValue.actual} string length found larger than the expected ${this.traceValue.expected} length`;
                }
                break;
            }
            case 'minimumExclusive': {
                if (this.traceValue.negated) {
                    explanation = `${this.traceValue.actual} string length found larger than the expected ${this.traceValue.expected} length`;
                }
                else {
                    explanation = `${this.traceValue.actual} string length found smaller or equal than the expected ${this.traceValue.expected} length`;
                }
                break;
            }
            case 'minimumInclusive': {
                if (this.traceValue.negated) {
                    explanation = `${this.traceValue.actual} string length found larger or equal than the expected ${this.traceValue.expected} length`;
                }
                else {
                    explanation = `${this.traceValue.actual} string length found smaller than the expected ${this.traceValue.expected} length`;
                }
                break;
            }
            case 'containsAll': {
                if (this.traceValue.negated) {
                    explanation = `all expected values included in found values ${this.traceValue.actual}`;
                }
                else {
                    const diff = this.difference(JSON.parse(this.traceValue.expected), JSON.parse(this.traceValue.actual));
                    explanation = `missing values ${JSON.stringify(diff)} in found values ${this.traceValue.actual}`;
                }
                break;
            }
            case 'containsSome': {
                if (this.traceValue.negated) {
                    const intersection = this.intersection(JSON.parse(this.traceValue.expected), JSON.parse(this.traceValue.actual));
                    explanation = `some unexpected values found ${intersection} in found values ${this.traceValue.actual}`;
                }
                else {
                    explanation = `none of the expected values found in found values ${this.traceValue.actual}`;
                }
                break;
            }
            case 'in': {
                if (this.traceValue.negated) {
                    explanation = `one of the non expected values found '${this.traceValue.actual}'`;
                }
                else {
                    explanation = `unexpected value found '${this.traceValue.actual}'`;
                }
                break;
            }
            case 'lessThan': {
                if (this.traceValue.negated) {
                    explanation = `value found '${this.traceValue.actual}' should not be less than to value '${this.traceValue.expected}' for property '${this.traceValue.altPath}'`;
                }
                else {
                    explanation = `value found '${this.traceValue.actual}' is greater than or equal to value '${this.traceValue.expected}' for property '${this.traceValue.altPath}'`;
                }
                break;
            }
            case 'lessThanOrEqual': {
                if (this.traceValue.negated) {
                    explanation = `value found '${this.traceValue.actual}' should not be less than or equal to value '${this.traceValue.expected}' for property '${this.traceValue.altPath}'`;
                }
                else {
                    explanation = `value found '${this.traceValue.actual}' is greater than value '${this.traceValue.expected}' for property '${this.traceValue.altPath}'`;
                }
                break;
            }
            case 'equalsTo': {
                if (this.traceValue.negated) {
                    explanation = `value found '${this.traceValue.actual}' should not be equal to value '${this.traceValue.expected}' for property '${this.traceValue.altPath}'`;
                }
                else {
                    explanation = `value found '${this.traceValue.actual}' is different to value '${this.traceValue.expected}' for property '${this.traceValue.altPath}'`;
                }
                break;
            }
            case 'disjointWith': {
                if (this.traceValue.negated) {
                    explanation = `value found '${this.traceValue.actual}' should not be different than value '${this.traceValue.expected}' for property '${this.traceValue.altPath}'`;
                }
                else {
                    explanation = `value found '${this.traceValue.actual}' is equal to value '${this.traceValue.expected}' for property '${this.traceValue.altPath}'`;
                }
                break;
            }
            case 'rego': {
                if (this.traceValue.negated) {
                    explanation = 'rego custom code didn\'t failed as expected';
                }
                else {
                    explanation = 'rego custom code failed unexpectedly';
                }
                break;
            }
            case 'datatype': {
                if (this.traceValue.negated) {
                    explanation = `value [${this.traceValue.actual}] unpexpectedly matches the datatype`;
                }
                else {
                    explanation = `value [${this.traceValue.actual}] does not match the datatype`;
                }
                break;
            }
            default: {
                if (this.traceValue.negated) {
                    explanation = `not (${this.traceValue.expected} ${this.traceValue.condition} ${this.traceValue.actual})`;
                }
                else {
                    explanation = `${this.traceValue.expected} ${this.traceValue.condition} ${this.traceValue.actual}`;
                }
                break;
            }
        }
        return explanation;
    }
    print(conformance, indentation, isLastConstraint = false) {
        const constraintIndex = isLastConstraint ? `${indentation}└──` : `${indentation}├──`;
        const toPrint = `(failure) ${this.invocation} => ${this.explanation}`;
        return `${constraintIndex}${colors_1.default.red(toPrint)}`;
    }
    intersection(a, b) {
        return a.filter((n) => {
            return b.includes(n);
        });
    }
    difference(a, b) {
        return a.filter((n) => {
            return !b.includes(n);
        });
    }
}
exports.Constraint = Constraint;
//# sourceMappingURL=constraint.js.map